/* gweather.c - Overall weather server functions
 *
 * SPDX-FileCopyrightText: The GWeather authors
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

#include "config.h"

#include "gweather.h"

/*
 * There is no code here: the purpose of this header is so that
 * gnome-builder knows how to pick up the include flags to parse
 * gweather.h properly.
 */
