<!--
SPDX-FileCopyrightText: 2021 GNOME Foundation

SPDX-License-Identifier: Apache-2.0 OR GPL-3.0-or-later
-->

{% extends "base.html" %}

{% block title %}{{ namespace.name }}.{{ class.name }}.{{ type_func.name }}{% endblock %}

{% block meta_other %}
  <meta property="og:title" content="{{ namespace.name }}.{{ class.name }}.{{ type_func.name }}"/>
  <meta property="og:description" content="Reference for {{ namespace.name }}.{{ class.name }}.{{ type_func.name }}"/>
  <meta name="twitter:title" content="{{ namespace.name }}.{{ class.name }}.{{ type_func.name }}"/>
  <meta name="twitter:description" content="Reference for {{ namespace.name }}.{{ class.name }}.{{ type_func.name }}"/>
{% endblock %}

{% block sidebar %}
<div class="section">
  <h5>Type</h5>
  <div class="links">
    <a href="{{ class.link_prefix }}.{{ class.name }}.html">{{ class.name }}</a>
  </div>
</div>
<div class="section">
  <h5>Constructors</h5>
  <div class="links">
  {% for m in class.ctors %}
    <a class="method{% if type_func.name == m.name %} current{% endif %}" href="ctor.{{ class.name }}.{{ m.name }}.html">{{ m.name }}</a>
  {% endfor %}
  </div>
</div>
{% endblock %}

{% block content %}
<section id="main" class="content">
  <header>
    <h3>Constructor</h3>
    <h1><a href="index.html">{{ namespace.name }}</a><span class="sep"></span><a href="{{ class.link_prefix }}.{{class.name}}.html">{{ class.name }}</a><span class="sep"></span>{{ type_func.name }}</h1>
  </header>

  <section>
    <div class="declaration toggle-wrapper">
      <h4 id="declaration" style="display:flex;">
        Declaration
        <a href="#declaration" class="anchor"></a>
        {% if CONFIG.source_location_url and type_func.source_location %}
        <a class="srclink" title="go to source location" href="{{ CONFIG.source_link(type_func.source_location) }}">[src]</a>
        {% endif %}
      </h4>

      <div class="docblock">
        <pre><code>{{ type_func.c_decl }}</code></pre>
      </div>
    </div>

    <div class="description toggle-wrapper">
      <h4 id="description" style="display:flex;">
        Description
        <a href="#description" class="anchor"></a>
        {% if CONFIG.source_location_url and type_func.docs_location %}
        <a class="srclink" title="go to source location" href="{{ CONFIG.source_link(type_func.docs_location) }}">[src]</a>
        {% endif %}
      </h4>

      <div class="docblock">
        {{ type_func.description }}
      </div>

      <div class="docblock">
        <table>
          {% if type_func.stability %}
          <tr><td>Stability:</td><td>{{ type_func.stability }}</td></tr>
          {% endif %}
          {% if type_func.available_since %}
          <tr><td>Available since:</td><td>{{ type_func.available_since }}</td></tr>
          {% endif %}
          {% if type_func.deprecated_since %}
          <tr><td>Deprecated since:</td><td>{{ type_func.deprecated_since.version }}</td></tr>
          <tr><td colspan="2">{{ type_func.deprecated_since.message }}</td></tr>
          {% endif %}
        </table>
      </div>

      {% if type_func.attributes %}
      <div class="docblock">
        <table>
        {% for (key, value) in type_func.attributes.items() %}
          <tr><td>{{ key|escape }}</td><td>{{ value|escape }}</td></tr>
        {% endfor %}
        </table>
      </div>
      {% endif %}
    </div>

    {% if type_func.arguments|length != 0 %}
    <div class="parameters toggle-wrapper">
      <h4 id="parameters">
        Parameters
        <a href="#parameters" class="anchor"></a>
      </h4>

      <div class="docblock">
        <table class="arguments">
          {% for arg in type_func.arguments %}
          <tr class="arg-name">
            <td><code>{{ arg.name }}</code></td>
            <td>
            {%- if arg.is_array -%}An array of&nbsp;{%- endif -%}
            {%- if arg.is_list -%}A list of&nbsp;{%- endif -%}
            {%- if arg.link -%}
            {{ arg.link|safe }}
            {%- else -%}
            <code>{{ arg.type_cname }}</code>
            {%- endif -%}
            </td>
          </tr>
          <tr class="arg-description">
            <td>&nbsp;</td>
            <td>{{ arg.description|safe }}</td>
          </tr>
          {% if arg.direction != "in" %}<tr><td>&nbsp;</td><td title="direction: {{ arg.direction}}">{{ arg.direction_note }}</td></tr>{% endif %}
          {% if arg.direction == "in" and arg.nullable %}<tr><td>&nbsp;</td><td>The argument can be <code>NULL</code>.</td></tr>{% endif %}
          {% if arg.direction in ["inout", "out"] and arg.nullable %}<tr><td>&nbsp;</td><td>The argument can be set to <code>NULL</code>.</td></tr>{% endif %}
          {% if arg.optional %}<tr><td>&nbsp;</td><td>The argument can be <code>NULL</code>.</td></tr>{% endif %}
          {% if arg.is_array and arg.zero_terminated %}<tr><td>&nbsp;</td><td>The array must be <code>NULL</code>-terminated.</td></tr>{% endif %}
          {% if arg.is_array and arg.fixed_size > 0 %}<tr><td>&nbsp;</td><td>The array must have {{ arg.fixed_size }} elements.</td></tr>{% endif %}
          {% if arg.is_array and arg.len_arg %}<tr><td>&nbsp;</td><td>The length of the array is specified in the <code>{{ arg.len_arg }}</code> argument.</td></tr>{% endif %}
          {% if arg.is_pointer %}<tr><td>&nbsp;</td><td title="transfer: {{ arg.transfer }}">{{ arg.transfer_note }}</td></tr>{% endif %}
          {% if arg.string_note %}<tr><td>&nbsp;</td><td>{{ arg.string_note }}</td></tr>{% endif %}
          {% endfor %}
          {% if type_func.throws %}
          <tr class="arg-name">
            <td><code>error</code></td>
            <td><code>GError **</code></td>
          </tr>
          <tr class="arg-description">
            <td>&nbsp;</td>
            <td>The return location for a <code>GError*</code>, or <code>NULL</code>.</td>
          </tr>
          {% endif %}
        </table>
      </div>
    </div>
    {% endif %}

    {% if type_func.return_value %}
    <div class="return-value toggle-wrapper">
      <h4 id="return-value">
        Return value
        <a href="#return-value" class="anchor"></a>
      </h4>

      <div class="docblock">
        <table class="returns">
          <tr class="arg-name">
            <td>Returns:</td>
            <td>
            {%- if type_func.return_value.is_array -%}An array of&nbsp;{%- endif -%}
            {%- if type_func.return_value.is_list -%}A list of&nbsp;{%- endif -%}
            {%- if type_func.return_value.link -%}
            {{ type_func.return_value.link|safe }}
            {%- else -%}
            <code>{{ type_func.return_value.type_cname }}</code>
            {%- endif -%}
            </td>
          </tr>
          <tr class="arg-description">
            <td>&nbsp;</td>
            <td>{{ type_func.return_value.description|safe }}</td>
          </tr>
          {% if type_func.return_value.is_array and type_func.return_value.zero_terminated %}<tr><td>&nbsp;</td><td>The array is <code>NULL</code>-terminated.</td></tr>{% endif %}
          {% if type_func.return_value.is_array and type_func.return_value.fixed_size > 0 %}<tr><td>&nbsp;</td><td>The array has {{ type_func.return_value.fixed_size }} elements.</td></tr>{% endif %}
          {% if type_func.return_value.is_array and type_func.return_value.len_arg %}<tr><td>&nbsp;</td><td>The length of the array is in the <code>{{ type_func.return_value.len_arg }}</code> argument.</td></tr>{% endif %}
          {% if type_func.return_value.is_pointer %}<tr><td>&nbsp;</td><td title="transfer: {{ type_func.return_value.transfer }}">{{ type_func.return_value.transfer_note }}</td></tr>{% endif %}
          {% if type_func.return_value.is_pointer and type_func.return_value.nullable %}<tr><td>&nbsp;</td><td>The return value can be <code>NULL</code>.</td></tr>{% endif %}
          {% if type_func.return_value.string_note %}<tr><td>&nbsp;</td><td>{{ type_func.return_value.string_note }}</td></tr>{% endif %}
        </table>
      </div>
    </div>
    {% endif %}
  </section>
</section>
{% endblock %}
